/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(crop_matrix_vcvc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(a6b385ed23d7afc2f53f2bb64577e004)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <radar/crop_matrix_vcvc.h>
// pydoc.h is automatically generated in the build directory
#include <crop_matrix_vcvc_pydoc.h>

void bind_crop_matrix_vcvc(py::module& m)
{

    using crop_matrix_vcvc = ::gr::radar::crop_matrix_vcvc;


    py::class_<crop_matrix_vcvc,
               gr::tagged_stream_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<crop_matrix_vcvc>>(
        m, "crop_matrix_vcvc", D(crop_matrix_vcvc))

        .def(py::init(&crop_matrix_vcvc::make),
             py::arg("vlen"),
             py::arg("crop_x"),
             py::arg("crop_y"),
             py::arg("len_key") = "packet_len",
             D(crop_matrix_vcvc, make))


        ;
}
