/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(estimator_ofdm.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(0150fe65cb971f8faefde835dcf6efc5)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <radar/estimator_ofdm.h>
// pydoc.h is automatically generated in the build directory
#include <estimator_ofdm_pydoc.h>

void bind_estimator_ofdm(py::module& m)
{

    using estimator_ofdm = ::gr::radar::estimator_ofdm;


    py::class_<estimator_ofdm,
               gr::block,
               gr::basic_block,
               std::shared_ptr<estimator_ofdm>>(m, "estimator_ofdm", D(estimator_ofdm))

        .def(py::init(&estimator_ofdm::make),
             py::arg("symbol_x"),
             py::arg("len_x"),
             py::arg("axis_x"),
             py::arg("symbol_y"),
             py::arg("len_y"),
             py::arg("axis_y"),
             py::arg("merge_consecutive") = true,
             D(estimator_ofdm, make))


        ;
}
