/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(msg_gate.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(45217f19d5d2438fe8dd3dbb3d3da61f)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <radar/msg_gate.h>
// pydoc.h is automatically generated in the build directory
#include <msg_gate_pydoc.h>

void bind_msg_gate(py::module& m)
{

    using msg_gate = ::gr::radar::msg_gate;


    py::class_<msg_gate, gr::block, gr::basic_block, std::shared_ptr<msg_gate>>(
        m, "msg_gate", D(msg_gate))

        .def(py::init(&msg_gate::make),
             py::arg("keys"),
             py::arg("val_min"),
             py::arg("val_max"),
             D(msg_gate, make))


        ;
}
