/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(split_cc.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(d6e7f66005b2c459b77909102073f847)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <radar/split_cc.h>
// pydoc.h is automatically generated in the build directory
#include <split_cc_pydoc.h>

void bind_split_cc(py::module& m)
{

    using split_cc = ::gr::radar::split_cc;


    py::class_<split_cc,
               gr::tagged_stream_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<split_cc>>(m, "split_cc", D(split_cc))

        .def(py::init(&split_cc::make),
             py::arg("packet_num"),
             py::arg("packet_parts"),
             py::arg("len_key") = "packet_len",
             D(split_cc, make))


        ;
}
