/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(estimator_fmcw.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(7aa05f03af07fd1ff14a6801bb62df83)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <radar/estimator_fmcw.h>
// pydoc.h is automatically generated in the build directory
#include <estimator_fmcw_pydoc.h>

void bind_estimator_fmcw(py::module& m)
{

    using estimator_fmcw = ::gr::radar::estimator_fmcw;


    py::class_<estimator_fmcw,
               gr::block,
               gr::basic_block,
               std::shared_ptr<estimator_fmcw>>(m, "estimator_fmcw", D(estimator_fmcw))

        .def(py::init(&estimator_fmcw::make),
             py::arg("samp_rate"),
             py::arg("center_freq"),
             py::arg("sweep_freq"),
             py::arg("samp_up"),
             py::arg("samp_down"),
             py::arg("push_power"),
             D(estimator_fmcw, make))


        ;
}
